# -*- coding: utf-8 -*-
# @author: leesoar

"""postcard"""

POP3_ADDR = {
    "tom": "pop.tom.com",
    "gmail": "pop.gmail.com",
    "yahoo": "pop.mail.yahoo.com",
    "outlook": "pop-mail.outlook.com",
    "china": "pop.china.com",
    "21cn": "pop.21cn.com",
    "163": "pop.163.com",
    "126": "pop.126.com",
    "qq": "pop.qq.com",
    "sina": "pop3.sina.com.cn",
    "88": "pop.88.com",
}

SMTP_ADDR = {
    "tom": "smtp.tom.com",
    "gmail": "smtp.gmail.com",
    "yahoo": "smtp.mail.yahoo.com",
    "outlook": "smtp-mail.outlook.com",
    "china": "smtp.china.com",
    "21cn": "smtp.21cn.com",
    "163": "smtp.163.com",
    "126": "smtp.126.com",
    "qq": "smtp.qq.com",
    "sina": "smtp.sina.com.cn",
    "88": "smtp.88.com",
}

IMAP_ADDR = {
    "tom": "imap.tom.com",
    "88": "imap.88.com",
}



# -*- coding: utf-8 -*-
# @author: leesoar

"""IMAP client"""

import base64
import imaplib
from email.parser import Parser
from urllib.parse import unquote

from postcard import setting


__all__ = ["Imap"]


class Imap(object):
    def __init__(self):
        self.user = None
        self.pwd = None
        self._client = None
        self.imap_addr = None
        self.imap_port = None
        self.parser = Parser()

    def login(self, user, pwd, imap_addr=None, imap_port=None, ssl=False):
        """Login current mailbox"""
        self.user = user
        self.pwd = pwd
        self.imap_addr = imap_addr or self.imap_addr
        self.imap_port = imap_port or self.imap_port

        if not self.imap_addr:
            self.imap_addr = self._set_imap_addr_auto(user)

        if not self.imap_port:
            self.imap_port = imaplib.IMAP4_SSL_PORT if ssl else imaplib.IMAP4_PORT

        if ssl:
            self._client = imaplib.IMAP4_SSL(self.imap_addr, self.imap_port)
        else:
            self._client = imaplib.IMAP4(self.imap_addr, self.imap_port)
        self._client.login(self.user, self.pwd)
        return self._client

    def close(self):
        self._client.close()
        self._client.logout()

    @staticmethod
    def get_addr_map():
        return setting.IMAP_ADDR

    def _set_imap_addr_auto(self, user):
        domain = user.rsplit(".", maxsplit=1)[0].split("@")[-1]
        imap_addr = self.get_addr_map().get(domain)
        if imap_addr and not self.imap_addr:
            self.imap_addr = imap_addr
        return self.imap_addr

    def get_client(self):
        """Get Imap client"""
        return self._client

    def how_many(self) -> int:
        """Return mailbox's email count"""
        _, msg_nums = self.select()
        return int(msg_nums[0] or 0)

    def _retr_num(self, msg_nums: list) -> list:
        return msg_nums[0].split()

    def select(self, mailbox="INBOX", readonly=True):
        return self._client.select(mailbox, readonly)

    def search(self, word, type_="Text", mailbox="INBOX", readonly=True):
        self.select(mailbox, readonly=readonly)
        _, msg_nums = self._client.search(None, type_, word)
        return self._retr_num(msg_nums)

    def unseen(self, readonly=True):
        self.select(readonly=readonly)
        return self.search(None, "Unseen")

    def __check_and_decode(self, mail_info: dict) -> dict:
        """Check encoding and decode mail's content"""
        content_encoding = mail_info.get("content-transfer-encoding", "").lower()

        try:
            if content_encoding == "base64":
                mail_info["content"] = base64.b64decode(mail_info["content"]).decode(errors="ignore")
            elif content_encoding == "quoted-printable":
                pass
        except Exception:
            pass

        return mail_info

    def retrieve(self, index_: int = None, mailbox="INBOX", *, readonly=True) -> dict:
        """Retrieve mail at index

        Tips:
            If you want to mark as read, please set `readonly=False`.
        """
        self.select(mailbox, readonly)
        index_ = index_ or self.how_many()

        if readonly is False:
            self._store(index_, flag_list="\\Seen")

        lines = self._client.fetch(str(index_).encode(), "RFC822")[1][0][1]
        lines = unquote(lines.decode())
        try:
            lines, content = lines.split("\r\n\r\n", maxsplit=1)
        except ValueError:
            content = ""
        finally:
            lines = lines.encode()

        line_array = lines.split(b"\r\n")

        def __convert(x: bytes):
            k, v = x.strip().decode(errors="ignore").split(":", 1)
            if "</p>" in v:
                k = "content"

            if " " in k:
                return
            return [k.lower().strip(), v.lower().strip().lstrip("[").rstrip("]")]

        mail_info = dict(filter(lambda x: x, map(__convert, filter(lambda x: b":" in x and b"receive" not in x.lower(), line_array))))

        if "content" not in mail_info:
            mail_info.update(dict(content=content))

        return self.__check_and_decode(mail_info)

    def _store(self, index_: int, command="+FLAGS", flag_list="\\Seen", readonly=False):
        self.select(readonly=False)
        self._client.store(str(index_).encode(), command, flag_list)
        self._client.expunge()
        self.select(readonly=readonly)

    def delete(self, index_: int, readonly=True):
        """Delete mail at index"""
        self._store(index_, flag_list="\\Deleted", readonly=readonly)

    def process(self, user=None, pwd=None, imap_addr=None, imap_port=None, ssl=False):
        """process decorator

        Args:
            user: mailbox account
            pwd: mailbox password
            imap_addr: current mailbox's Imap addr
            imap_port: current mailbox's Imap port
            ssl: use SSL, default False
        Usage:
            imap = Imap()

            @imap.process(user="xxx", pwd="xxx")
            def get_content():
                content = imap.retrieve()["content"]
                log.debug(content)
        """
        user = user or self.user
        pwd = pwd or self.pwd
        imap_addr = imap_addr or self.imap_addr
        imap_port = imap_port or self.imap_port

        assert None not in [user, pwd], \
            "You should give your info."

        def decorator(func):
            def wrapper(*args, **kwargs):
                self.login(user, pwd, imap_addr, imap_port, ssl)
                ret = func(*args, **kwargs)
                self.close()
                return ret
            return wrapper
        return decorator



# -*- coding: utf-8 -*-
# @author: leesoar

"""SMTP client"""
import os
import smtplib
from email.header import Header
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.utils import formataddr

from postcard import setting

__all__ = ["Smtp"]


class Smtp(object):
    def __init__(self, debug=False):
        self.user = None
        self.pwd = None
        self._client = None
        self.smtp_addr = None
        self.smtp_port = None
        self.nickname = self.user
        self._debug = debug

    def login(self, user, pwd, smtp_addr=None, smtp_port=None, ssl=True):
        """Login current mailbox"""
        self.user = user
        self.pwd = pwd
        self.smtp_addr = smtp_addr or self.smtp_addr
        self.smtp_port = smtp_port or self.smtp_port

        if not self.smtp_addr:
            self.smtp_addr = self._set_smtp_addr_auto(user)

        if not self.smtp_port:
            self.smtp_port = smtplib.SMTP_SSL_PORT if ssl else smtplib.SMTP_PORT

        if ssl:
            self._client = smtplib.SMTP_SSL(self.smtp_addr, self.smtp_port)
        else:
            self._client = smtplib.SMTP(self.smtp_addr, self.smtp_port)
        self._client.login(self.user, self.pwd)
        return self._client

    def close(self):
        self._client.quit()

    def set_nickname(self, nickname):
        self.nickname = nickname

    @staticmethod
    def get_addr_map():
        return setting.SMTP_ADDR

    def _set_smtp_addr_auto(self, user):
        domain = user.rsplit(".", maxsplit=1)[0].split("@")[-1]
        smtp_addr = self.get_addr_map().get(domain)
        if smtp_addr and not self.smtp_addr:
            self.smtp_addr = smtp_addr
        return self.smtp_addr

    def get_client(self):
        """Get smtp client"""
        return self._client

    def _send(self, msg: MIMEText or MIMEMultipart, receiver):
        try:
            self._client.set_debuglevel(self._debug)
            self._client.ehlo()
            self._client.sendmail(self.user, receiver, msg.as_string())
        except smtplib.SMTPException:
            return 0
        else:
            return 1

    @staticmethod
    def _review_receiver(receiver):
        if isinstance(receiver, str):
            receiver = [receiver]
        return receiver

    def send_mail(self, subject, content, receiver, subtype="html", charset="utf-8"):
        msg = MIMEText(content, subtype, charset)
        msg["Subject"] = Header(subject, charset)
        msg["From"] = formataddr([self.nickname, self.user], charset)

        msg["To"] = ",".join(self._review_receiver(receiver))

        return self._send(msg, receiver)

    def send_mail_with_attachment(self, subject, content, receiver, *file_paths, subtype="html", charset="utf-8"):
        msg = MIMEMultipart()
        msg["Subject"] = Header(subject, charset)
        msg["From"] = formataddr([self.nickname, charset], charset=charset)
        msg["To"] = ",".join(self._review_receiver(receiver))
        msg.attach(MIMEText(content, subtype, charset))

        for file_path in file_paths:
            filename = os.path.split(file_path)[-1]
            file = MIMEText(open(file_path, "rb").read(), "base64", charset)
            file["Content-Type"] = "application/octet-stream"
            file["Content-Disposition"] = f'attachment; filename="{filename}"'
            msg.attach(file)

        return self._send(msg, receiver)

    def process(self, user=None, pwd=None, smtp_addr=None, smtp_port=None, ssl=False):
        """process decorator

        Args:
            user: mailbox account
            pwd: mailbox password
            smtp_addr: current mailbox's smtp addr
            smtp_port: current mailbox's smtp port
            ssl: use SSL, default False
        Usage:
            smtp = Smtp()

            @smtp.process(user="xxx", pwd="xxx")
            def get_content():
                ret = smtp.send_mail("xxx", "xxx", "xxx")
                log.debug(ret)
        """
        user = user or self.user
        pwd = pwd or self.pwd
        smtp_addr = smtp_addr or self.smtp_addr
        smtp_port = smtp_port or self.smtp_port

        assert None not in [user, pwd], \
            "You should give your info."

        def decorator(func):
            def wrapper(*args, **kwargs):
                self.login(user, pwd, smtp_addr, smtp_port, ssl)
                ret = func(*args, **kwargs)
                self.close()
                return ret
            return wrapper
        return decorator
